package cn.fastfun.service.entity;


import cn.fastfun.controller.param.DeviceFormParam;
import cn.fastfun.util.ObjectUtil;

import com.bridge.entity.DateEntity;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;

import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import org.hibernate.annotations.JoinColumnOrFormula;
import org.hibernate.annotations.JoinColumnsOrFormulas;
import org.hibernate.annotations.JoinFormula;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;

import java.util.Date;
import java.util.Map;


@Entity
@Table(name = "app_device")
@Setter
@Getter
@NoArgsConstructor
public class AppDevice extends DateEntity {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "SN号", name = "sn", required = true)
    @Column(name = "sn")
    private String sn;

    @ApiModelProperty(value = "操作号", name = "operateID", required = true)
    @Column(name = "operate_id")
    private Integer operateID;

    @ApiModelProperty(value = "IMEI", name = "imei", required = true)
    @Column(name = "imei")
    private String imei;

    @ApiModelProperty(value = "批次号", name = "batchNum", required = true, example = "")
    @Column(name = "batch_num")
    private String batchNum;

    @ApiModelProperty(value = "收货地", name = "received_place", required = true, example = "")
    @Column(name = "received_place")
    private String receivedPlace;

    @ApiModelProperty(value = "状态", name = "status", required = true)
    @Column(name = "status")
    private Integer status = 0;

    // 关联
    @JsonIgnore
    @ManyToOne(targetEntity = SysDict.class)
    @JoinColumnsOrFormulas(value = {
            @JoinColumnOrFormula(column = @JoinColumn(name = "status", referencedColumnName = "name", insertable = false, updatable = false, nullable = false)),
            @JoinColumnOrFormula(formula = @JoinFormula(value = "'app_device_status'", referencedColumnName = "group_name"))
    })
    private SysDict statusDict;

    @Transient
    @JsonInclude(value = JsonInclude.Include.NON_NULL)
    public String getStatusTitle() {
        return null != statusDict ? statusDict.getValue() : "";
    }

    @ApiModelProperty(value = "操作者", name = "operator", required = true)
    @Column(name = "operator")
    private String operator;

    @ApiModelProperty(value = "发货时间", name = "deliverTime", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd ")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @Column(name = "deliver_time")
    private Date deliverTime;

    @ApiModelProperty(value = "入库时间", name = "transferTime", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @Column(name = "instorage_time")
    private Date instorageTime;

    @ApiModelProperty(value = "最新调拨时间", name = "transferTime", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @Column(name = "transfer_time")
    private Date transferTime;

    @ApiModelProperty(value = "最新回调时间", name = "transferBackTime", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @Column(name = "transferback_time")
    private Date transferBackTime;

    @ApiModelProperty(value = "最新出库时间", name = "outstorageTime", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @Column(name = "outstorage_time")
    private Date outstorageTime;

    @ApiModelProperty(value = "最新处理时间", name = "handleTime", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @Column(name = "handle_time")
    private Date handleTime;


    @ApiModelProperty(value = "自检状态", name = "checkStatus", required = true)
    @Column(name = "check_status")
    private Integer checkStatus = 0;

    @ApiModelProperty(value = "自检数据连接", name = "checkStatusDataConnect", required = true)
    @Column(name = "self_check_data_connect")
    private Integer checkStatusDataConnect = 0;

    @ApiModelProperty(value = "自检定位", name = "checkStatusLocation", required = true)
    @Column(name = "self_check_location")
    private Integer checkStatusLocation = 0;


    @ApiModelProperty(value = "自检上锁", name = "checkStatusLock", required = true)
    @Column(name = "self_check_lock")
    private Integer checkStatusLock = 0;


    @ApiModelProperty(value = "自检故障", name = "checkStatusFault", required = true)
    @Column(name = "self_check_fault")
    private Integer checkStatusFault = 0;


    @ApiModelProperty(value = "自检电压", name = "checkStatusVoltage", required = true)
    @Column(name = "self_check_voltage")
    private Integer checkStatusVoltage = 0;


    // 关联
    @JsonIgnore
    @ManyToOne(targetEntity = SysDict.class)
    @JoinColumnsOrFormulas(value = {
            @JoinColumnOrFormula(column = @JoinColumn(name = "check_status", referencedColumnName = "name", insertable = false, updatable = false, nullable = false)),
            @JoinColumnOrFormula(formula = @JoinFormula(value = "'app_device_check_status'", referencedColumnName = "group_name"))
    })
    private SysDict checkStatusDict;

    @Transient
    @JsonInclude(value = JsonInclude.Include.NON_NULL)
    public String getCheckStatusTitle() {
        return null != checkStatusDict ? checkStatusDict.getValue() : "";
    }


    @ApiModelProperty(value = "归属", name = "ownerId", required = true)
    @Column(name = "owner_id")
    private String ownerId;


    public AppDevice(Map<String, Object> param) {
        setBatchNum(ObjectUtil.obj2String(param.get("pathId")));
        setSn(ObjectUtil.obj2String(param.get("sn")));
        setImei(ObjectUtil.obj2String(param.get("imei")));
        setDeliverTime(ObjectUtil.obj2Date(param.get("deliverTime")));
        setReceivedPlace(ObjectUtil.obj2String(param.get("receivedPlace")));
        setCheckStatus(0);
        setStatus(0);
    }

    // 类型
    public String getType() {
        return imei.substring(0, 1);
    }

    public String getPack() {
        return imei.substring(1, 3);
    }

    public String getChemistry() { // 化学体系
        return imei.substring(5, 6);
    }

    public String getPlatform() { // 电池平台
        return imei.substring(6, 7);
    }

    public String getCapacity() {// 容量
        return imei.substring(7, 9);
    }

    public String getExpand() {// 容量
        return imei.substring(9, 10);
    }
    // 显示名称
    @Transient
    private String typeTitle = "";// 产品类型

    @Transient
    private String packTitle = ""; // PACK制造商

    @Transient
    private String specTitle = ""; // 产品规格

    @Transient
    private String expandTitle = ""; // 产品扩展


    /**
     * 从参数转为实体
     */
    public AppDevice fromFormParam(DeviceFormParam param) {
        setSn(param.getSn());
        setImei(param.getImei());
        setBatchNum(param.getBatchNum());
        setDeliverTime(param.getDeliverTime());
        setReceivedPlace(param.getReceivedPlace());
        setStatus(0);
        setCheckStatus(0);
        return this;
    }

    @ApiModelProperty(value = "用途", name = "tfUsed", required = true)
    @Column(name = "tf_used")
    private String tfUsed;


    @ApiModelProperty(value = "描述", name = "tfDescribe", required = true)
    @Column(name = "tf_describe")
    private String tfDescribe;
    public void appendInfo(AppDeviceLog deviceLog) {
        setTfUsed(deviceLog.getTfUsed());
        setTfDescribe(deviceLog.getTfDescribe());
    }
}
